import { CoreManagerInstance } from './CoreManager';
import ParsePromise from './ParsePromise';

export function makeDatonis(coreManager) {
  return class extends DatonisFacade {
    constructor(datonisToken, version) {
      super(datonisToken, version, coreManager);
    }
  };
}

export default class DatonisFacade {

  /**
   * @param {(String)} datonisToken Datonis auth token
   * @param {(Number)} version Datonis api version
   */

  /**
   * @property datonisToken
   * @type String
   */
  constructor(datonisToken, version, coreManager) {

    this._CoreManager = coreManager;

    this.XHR = null;
    if (typeof XMLHttpRequest !== 'undefined') {
      this.XHR = XMLHttpRequest;
    }


    if (typeof datonisToken === 'string') {
      this.datonisToken = datonisToken;
    } else {
      throw new TypeError('Datonis token must be of type string.');
    }
    version = version || 3;
    if (typeof version === 'number') {
      this.version = version;
    } else {
      throw new TypeError('Datonis version must be of type number.');
    }
  }

  /**
   * Get things.
   * @param {Hash} data
   * @param {Hash} options
   * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
   */

  /**
   * @property version
   * @type Number
   */
  getThings(data = {}, options = {}) {
    let url = this.getDatonisUrl() + 'api/v' + this.version + '/things';

    return this.ajax('GET', url, data, options);
  }

  /**
   * Get things.
   * @param {Hash} data
   * @param {Hash} options
   * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
   */
  getThingRawEvents(data = {}, options = {}) {
    let url = this.getDatonisUrl() + 'api/v' + this.version + '/datonis_query/thing_data';

    return this.ajax('POST', url, data, options);
  }

  /**
   * Get things.
   * @param {Hash} data
   * @param {Hash} options
   * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
   */
  getThingWaypoints(thing_key, data = {}, options = {}) {
    let url = this.getDatonisUrl() + 'api/v' + this.version + '/things/' + thing_key + '/waypoints';

    return this.ajax('GET', url, data, options);
  }

  /**
   * Get thing aggregated data. Function defination can be improved in future in case any additional computation required
   * from sdk.
   * @param {Hash} data
   * @param {Hash} options
   * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
   */
  getThingAggregateData(data = {}, options = {}) {
    let url = this.getDatonisUrl() + 'api/v' + this.version + '/datonis_query/thing_aggregated_data';

    return this.ajax('POST', url, data, options);
  }

  ajax(method, url, data, headers) {
    var _this = this;
    var promise = new ParsePromise();

    (function () {
      if (this.XHR == null) {
        throw new Error('Cannot make a request: No definition of XMLHttpRequest was found.');
      }
      var handled = false;
      const XHR = this.XHR;
      var xhr = new XHR();

      xhr.onreadystatechange = function () {
        if (xhr.readyState !== 4 || handled) {
          return;
        }
        handled = true;

        if (xhr.status >= 200 && xhr.status < 300) {
          var response;
          try {
            response = JSON.parse(xhr.responseText);
          } catch (e) {
            promise.reject(e.toString());
          }
          if (response) {
            promise.resolve(response, xhr.status, xhr);
          }
        } else {
          promise.reject(xhr);
        }
      };

      headers = headers || {};
      headers['X-Auth-Token'] = _this.datonisToken;
      if (typeof headers['Content-Type'] !== 'string') {
        headers['Content-Type'] = 'application/json'; // Avoid pre-flight
      }

      xhr.open(method, url, true);
      for (var h in headers) {
        xhr.setRequestHeader(h, headers[h]);
      }
      xhr.send(data);
    })();

    return promise;
  }

  getDatonisUrl() {
    var datonisUrl = this._CoreManager.get('DATONIS_URL');
    if (datonisUrl[datonisUrl.length - 1] !== '/') {
      datonisUrl += '/';
    }
    return datonisUrl;
  }
}