/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

import { CoreManager, CoreManagerInstance } from './CoreManager';
import decode from './decode';
import encode from './encode';
import escape from './escape';
import ParseError from './ParseError';
import ParsePromise from './ParsePromise';

/**
 * Parse.Config is a local representation of configuration data that
 * can be set from the Parse dashboard.
 *
 * @alias Parse.Config
 */

class ParseConfig {

  constructor() {
    this.attributes = {};
    this._escapedAttributes = {};
  }

  /**
   * Gets the value of an attribute.
   * @param {String} attr The name of an attribute.
   */
  get(attr) {
    return this.attributes[attr];
  }

  /**
   * Gets the HTML-escaped value of an attribute.
   * @param {String} attr The name of an attribute.
   */
  escape(attr) {
    var html = this._escapedAttributes[attr];
    if (html) {
      return html;
    }
    var val = this.attributes[attr];
    var escaped = '';
    if (val != null) {
      escaped = escape(val.toString());
    }
    this._escapedAttributes[attr] = escaped;
    return escaped;
  }
}

export class ParseConfigFacade {

  constructor(coreManager) {
    this._CoreManager = coreManager;
    this._CoreManager.setConfigFacade(this);
    this._CoreManager.setConfigController(new DefaultController(this._CoreManager));
    this.Instance = ParseConfig;
  }

  /**
   * Retrieves the most recently-fetched configuration object, either from
   * memory or from local storage if necessary.
   *
   * @static
   * @return {Config} The most recently-fetched Parse.Config if it
   *     exists, else an empty Parse.Config.
   */
  current() {
    var controller = this._CoreManager.getConfigController();
    return controller.current();
  }

  /**
   * Gets a new configuration object from the server.
   * @static
   * @param {Object} options A Backbone-style options object.
   * Valid options are:<ul>
   *   <li>success: Function to call when the get completes successfully.
   *   <li>error: Function to call when the get fails.
   * </ul>
   * @return {Parse.Promise} A promise that is resolved with a newly-created
   *     configuration object when the get completes.
   */
  get(options) {
    options = options || {};

    var controller = this._CoreManager.getConfigController();
    return controller.get()._thenRunCallbacks(options);
  }
}

class DefaultController {

  constructor(coreManager) {
    this._CoreManager = coreManager;
    this.currentConfig = null;
    this.CURRENT_CONFIG_KEY = 'currentConfig';
  }

  current() {
    if (this.currentConfig) {
      return this.currentConfig;
    }

    var config = new ParseConfig();
    var storagePath = this._CoreManager.getStorageFacade().generatePath(this.CURRENT_CONFIG_KEY);
    var configData;
    if (!this._CoreManager.getStorageFacade().async()) {
      configData = this._CoreManager.getStorageFacade().getItem(storagePath);

      if (configData) {
        var attributes = this.decodePayload(configData);
        if (attributes) {
          config.attributes = attributes;
          this.currentConfig = config;
        }
      }
      return config;
    }
    // Return a promise for async storage controllers
    return this._CoreManager.getStorageFacade().getItemAsync(storagePath).then(configData => {
      if (configData) {
        var attributes = this.decodePayload(configData);
        if (attributes) {
          config.attributes = attributes;
          this.currentConfig = config;
        }
      }
      return config;
    });
  }

  decodePayload(data) {
    try {
      var json = JSON.parse(data);
      if (json && typeof json === 'object') {
        return decode(json, this._CoreManager);
      }
    } catch (e) {
      return null;
    }
  }

  get() {
    var RESTController = this._CoreManager.getRESTController();

    return RESTController.request('GET', 'config', {}, {}).then(response => {
      if (!response || !response.params) {
        var error = new ParseError(ParseError.INVALID_JSON, 'Config JSON response invalid.');
        return ParsePromise.error(error);
      }

      var config = new ParseConfig();
      config.attributes = {};
      for (var attr in response.params) {
        config.attributes[attr] = decode(response.params[attr], this._CoreManager);
      }
      this.currentConfig = config;
      return this._CoreManager.getStorageFacade().setItemAsync(this._CoreManager.getStorageFacade().generatePath(this.CURRENT_CONFIG_KEY), JSON.stringify(response.params)).then(() => {
        return config;
      });
    });
  }
}

const _ParseConfigFacade = new ParseConfigFacade(CoreManager);

export default ParseConfig;