/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

import * as ObjectStateMutations from './ObjectStateMutations';
import TaskQueue from './TaskQueue';

export default class UniqueInstanceStateController {
  constructor() {
    this.objectState = new WeakMap();
  }

  getState(obj) {
    let classData = this.objectState.get(obj);
    return classData || null;
  }

  initializeState(obj, initial) {
    let state = this.getState(obj);
    if (state) {
      return state;
    }
    if (!initial) {
      initial = {
        serverData: {},
        pendingOps: [{}],
        objectCache: {},
        tasks: new TaskQueue(),
        existed: false
      };
    }
    state = initial;
    this.objectState.set(obj, state);
    return state;
  }

  removeState(obj) {
    let state = this.getState(obj);
    if (state === null) {
      return null;
    }
    this.objectState.delete(obj);
    return state;
  }

  getServerData(obj) {
    let state = this.getState(obj);
    if (state) {
      return state.serverData;
    }
    return {};
  }

  setServerData(obj, attributes) {
    let serverData = this.initializeState(obj).serverData;
    ObjectStateMutations.setServerData(serverData, attributes);
  }

  getPendingOps(obj) {
    let state = this.getState(obj);
    if (state) {
      return state.pendingOps;
    }
    return [{}];
  }

  setPendingOp(obj, attr, op) {
    let pendingOps = this.initializeState(obj).pendingOps;
    ObjectStateMutations.setPendingOp(pendingOps, attr, op);
  }

  pushPendingState(obj) {
    let pendingOps = this.initializeState(obj).pendingOps;
    ObjectStateMutations.pushPendingState(pendingOps);
  }

  popPendingState(obj) {
    let pendingOps = this.initializeState(obj).pendingOps;
    return ObjectStateMutations.popPendingState(pendingOps);
  }

  mergeFirstPendingState(obj) {
    let pendingOps = this.getPendingOps(obj);
    ObjectStateMutations.mergeFirstPendingState(pendingOps);
  }

  getObjectCache(obj) {
    let state = this.getState(obj);
    if (state) {
      return state.objectCache;
    }
    return {};
  }

  estimateAttribute(obj, attr) {
    let serverData = this.getServerData(obj);
    let pendingOps = this.getPendingOps(obj);
    return ObjectStateMutations.estimateAttribute(serverData, pendingOps, obj.className, obj.id, obj.coreManager, attr);
  }

  estimateAttributes(obj) {
    let serverData = this.getServerData(obj);
    let pendingOps = this.getPendingOps(obj);
    return ObjectStateMutations.estimateAttributes(serverData, pendingOps, obj.className, obj.id, obj.coreManager);
  }

  commitServerChanges(obj, changes) {
    let state = this.initializeState(obj);
    ObjectStateMutations.commitServerChanges(state.serverData, state.objectCache, changes);
  }

  enqueueTask(obj, task) {
    let state = this.initializeState(obj);
    return state.tasks.enqueue(task);
  }

  duplicateState(source, dest) {
    let oldState = this.initializeState(source);
    let newState = this.initializeState(dest);
    for (let key in oldState.serverData) {
      newState.serverData[key] = oldState.serverData[key];
    }
    for (let index = 0; index < oldState.pendingOps.length; index++) {
      for (let key in oldState.pendingOps[index]) {
        newState.pendingOps[index][key] = oldState.pendingOps[index][key];
      }
    }
    for (let key in oldState.objectCache) {
      newState.objectCache[key] = oldState.objectCache[key];
    }
    newState.existed = oldState.existed;
  }

  clearAllState() {
    this.objectState = new WeakMap();
  }
}